==> kde4libs_3.80.1.orig.tar.gz: kde4libs-3.80.1/kio/kssl/ksslcsessioncache.cc <==
/* This file is part of the KDE project
 *
 * Copyright (C) 2003 Stefan Rompf <sux@loplof.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include <qpair.h>
#include <qstring.h>
#include <q3ptrlist.h>

#include <kdebug.h>
#include <kstaticdeleter.h>
#include <kurl.h>

#include "ksslconfig.h"
#include "ksslcsessioncache.h"

/* 
 * Operation:
 *
 * Sessions will be stored per running application, not KDE
 * wide, to avoid security problems with hostile programs
 * that negotiate sessions with weak cryptographic keys and store
 * them for everybody to use - I really don't want that.
 *
 * Retrieval is organized similar to George's thoughts in the KSSLD
 * certificate cache: The cache is organised as a list, with the
 * recently fetched (or stored) session first.
 *
 * The cache has an artificial limit of 32 sessions (should really
 * be enough), and relies on the peer server for timeouts
 *
 */
#define MAX_ENTRIES 32

#ifdef KSSL_HAVE_SSL

typedef QPair<QString,QString> KSSLCSession;
typedef Q3PtrList<KSSLCSession> KSSLCSessions;

static KSSLCSessions *sessions = 0L;
static KStaticDeleter<KSSLCSessions> med;


static QString URLtoKey(const KUrl &kurl) {
    return kurl.host() + ':' + kurl.protocol() + ':' + QString::number(kurl.port());
}


static void setup() {
    KSSLCSessions *ses = new KSSLCSessions;
    ses->setAutoDelete(true);
    med.setObject(sessions, ses);
}

#endif

QString KSSLCSessionCache::getSessionForURL(const KUrl &kurl) {
#ifdef KSSL_HAVE_SSL
    if (!sessions) return QString();
    QString key = URLtoKey(kurl);

    for(KSSLCSession *it = sessions->first(); it; it=sessions->next()) {
	if (it->first == key) {
	    sessions->take();
	    sessions->prepend(it);
	    return it->second;
	}
    }

    // Negative caching disabled: cache pollution
#if 0 
    kDebug(7029) <<"Negative caching " <<key <<endl;
    if (sessions->count() >= MAX_ENTRIES) sessions->removeLast();
    sessions->prepend(new KSSLCSession(key, QString()));
#endif

#endif
    return QString();
}


void KSSLCSessionCache::putSessionForURL(const KUrl &kurl, const QString &session) {
#ifdef KSSL_HAVE_SSL
    if (!sessions) setup();
    QString key = URLtoKey(kurl);
    KSSLCSession *it;

    for(it = sessions->first(); it && it->first != key; it=sessions->next());

    if (it) {
	sessions->take();
	it->second = session;
    } else {
	it = new KSSLCSession(key, session);
	if (sessions->count() >= MAX_ENTRIES) sessions->removeLast();
    }

    sessions->prepend(it);
#endif
}

==> kde4libs_3.80.3.orig.tar.gz: kde4libs-3.80.3/kio/kssl/ksslcsessioncache.cc <==
/* This file is part of the KDE project
 *
 * Copyright (C) 2003 Stefan Rompf <sux@loplof.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include <qpair.h>
#include <qstring.h>
#include <q3ptrlist.h>

#include <kdebug.h>
#include <kstaticdeleter.h>
#include <kurl.h>

#include "ksslconfig.h"
#include "ksslcsessioncache.h"

/*
 * Operation:
 *
 * Sessions will be stored per running application, not KDE
 * wide, to avoid security problems with hostile programs
 * that negotiate sessions with weak cryptographic keys and store
 * them for everybody to use - I really don't want that.
 *
 * Retrieval is organized similar to George's thoughts in the KSSLD
 * certificate cache: The cache is organised as a list, with the
 * recently fetched (or stored) session first.
 *
 * The cache has an artificial limit of 32 sessions (should really
 * be enough), and relies on the peer server for timeouts
 *
 */
#define MAX_ENTRIES 32

#ifdef KSSL_HAVE_SSL

typedef QPair<QString,QString> KSSLCSession;
typedef Q3PtrList<KSSLCSession> KSSLCSessions;

static KSSLCSessions *sessions = 0L;
static KStaticDeleter<KSSLCSessions> med;


static QString URLtoKey(const KUrl &kurl) {
    return kurl.host() + ':' + kurl.protocol() + ':' + QString::number(kurl.port());
}


static void setup() {
    KSSLCSessions *ses = new KSSLCSessions;
    ses->setAutoDelete(true);
    med.setObject(sessions, ses);
}

#endif

QString KSSLCSessionCache::getSessionForUrl(const KUrl &kurl) {
#ifdef KSSL_HAVE_SSL
    if (!sessions) return QString();
    QString key = URLtoKey(kurl);

    for(KSSLCSession *it = sessions->first(); it; it=sessions->next()) {
	if (it->first == key) {
	    sessions->take();
	    sessions->prepend(it);
	    return it->second;
	}
    }

    // Negative caching disabled: cache pollution
#if 0
    kDebug(7029) <<"Negative caching " <<key <<endl;
    if (sessions->count() >= MAX_ENTRIES) sessions->removeLast();
    sessions->prepend(new KSSLCSession(key, QString()));
#endif

#endif
    return QString();
}


void KSSLCSessionCache::putSessionForUrl(const KUrl &kurl, const QString &session) {
#ifdef KSSL_HAVE_SSL
    if (!sessions) setup();
    QString key = URLtoKey(kurl);
    KSSLCSession *it;

    for(it = sessions->first(); it && it->first != key; it=sessions->next());

    if (it) {
	sessions->take();
	it->second = session;
    } else {
	it = new KSSLCSession(key, session);
	if (sessions->count() >= MAX_ENTRIES) sessions->removeLast();
    }

    sessions->prepend(it);
#endif
}
